function[L,U,P,Q] = lud_fp(A)
% Computes the LU factorization on a square, invertible matrix A using full
% pivoting.
%
%  P A Q = L U

[m,n] = size(A);
assert(m == n);

L = eye(n);
U = A;
P = eye(n);
Q = eye(n);

for k = 1:(n-1)

  % Find pivot:
  [colmax,col_inds] = max(abs(U(k:n,k:n)));
  [rowmax,col_pivot] = max(colmax);
  row_pivot = col_inds(col_pivot);

  col_pivot = (k-1) + col_pivot;
  row_pivot = (k-1) + row_pivot;
  
  % Row pivoting: Interchange rows of U, L, and P
  U([k row_pivot],:) = U([row_pivot k], :);
  P([k row_pivot],:) = P([row_pivot k], :);
  L([k row_pivot],1:(k-1)) = L([row_pivot k], 1:(k-1));

  % Column pivoting: interchange columns of U and record in Q
  Q(:,[col_pivot k]) = Q(:,[k col_pivot]);
  U(:,[col_pivot k]) = U(:,[k col_pivot]);

  % record row operations
  L(k+1:n, k) = U(k+1:n,k)./U(k,k);
  
  % perform row operations
  U(k+1:n,:) = U(k+1:n,:) - L(k+1:n,k)*U(k,:);

end
