function[L,U,P] = lud_pp(A)
% Computes the LU factorization on a square, invertible matrix A using partial
% (row) pivoting.
%
%  P A = L U

[m,n] = size(A);
assert(m == n);

L = eye(n);
U = A;
P = eye(n);

for k = 1:(n-1)

  % Find pivot:
  [~,pivot_ind] = max(abs(U(k:n,k)));
  pivot_ind = (k-1) + pivot_ind;
  
  % Interchange rows of U, L, and P
  U([k pivot_ind],:) = U([pivot_ind k], :);
  P([k pivot_ind],:) = P([pivot_ind k], :);
  L([k pivot_ind],1:(k-1)) = L([pivot_ind k], 1:(k-1));

  % record row operations
  L(k+1:n, k) = U(k+1:n,k)./U(k,k);
  
  % perform row operations
  U(k+1:n,:) = U(k+1:n,:) - L(k+1:n,k)*U(k,:);

end
